/////////////////////////////////////////////////////////////////////////////
// morra.cpp: application implementation file



#include "stdafx.h"
#include "resource.h"

// Random functions support
#include "stdlib.h"
#include "stdio.h"
#include "time.h"

#include "morra.h"


static UINT BASED_CODE upperButtons[] =
{
	ID_GAME_EXIT,
	ID_GAME_RESTART,
		ID_SEPARATOR,
	ID_HELP_RULES,
	ID_HELP_ABOUT,
};

static UINT BASED_CODE lowerButtons[] =
{
		ID_SEPARATOR,
		ID_SEPARATOR,
	ID_PAPER,
	ID_SCISSORS,
	ID_STONE,
};





/////////////////////////////////////////////////////////////////////////////

// theApp:
// Just creating this application object runs the whole application.
//
CTheApp NEAR theApp;

/////////////////////////////////////////////////////////////////////////////
// CTheApp

// InitInstance:
// When any CTheApp object is created, this member function is automatically
// called.  Any data may be set up at this point.
//
// Also, the main window of the application should be created and shown here.
// Return TRUE if the initialization is successful.
//
BOOL CTheApp::InitInstance()
{
	SetDialogBkColor();     // hook grey dialogs (was default in MFC V1)

	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow(m_nCmdShow);
	m_pMainWnd->UpdateWindow();
	
	return TRUE;
}





/////////////////////////////////////////////////////////////////////////////

// CMainWindow constructor:
// Create the window with the appropriate style, size, menu, etc.
//
CMainWindow::CMainWindow()
{
	CRect wndRect(rectDefault);
	wndRect.right = wndRect.left + AREA_WIDTH;
	wndRect.bottom = wndRect.top + AREA_HEIGHT;

	// Menu Height
	wndRect.bottom += MENU_HEIGHT;
	
	CalcWindowRect(wndRect);

    CString appName;
    appName.LoadString(AFX_IDS_APP_TITLE);
	
	Create(NULL,
			appName,
			WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX,
			wndRect,
			NULL);
    
    // Create Window Font
	m_font.CreateFont(-TEXT_HEIGHT,0,0,0,FW_BOLD,0,0,0,DEFAULT_CHARSET,
						OUT_DEVICE_PRECIS,CLIP_DEFAULT_PRECIS,
						DEFAULT_QUALITY,FF_DONTCARE,TEXT_FONT);
 
    // Creating the Buttons
    // [HumanMove][Space][IBMMove][Response]
    wndRect.bottom = AREA_HEIGHT - BOTTOM_MARGIN;
    wndRect.top = wndRect.bottom - BUTTON_HEIGHT;
    wndRect.left = (AREA_WIDTH - 3*BUTTON_WIDTH - BUTTON_SPACING)/2;
    wndRect.right = wndRect.left + BUTTON_WIDTH;
    m_humanMove.Create("",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
    										wndRect,this,IDC_HUMAN);
	m_humanMove.LoadBitmaps("EMPTY");
    
    wndRect.left = wndRect.right + BUTTON_SPACING + 1;
    wndRect.right = wndRect.left + BUTTON_WIDTH;
    m_IBMMove.Create("",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
    										wndRect,this,IDC_IBM);
	m_IBMMove.LoadBitmaps("EMPTY");
    
    wndRect.left = wndRect.right + 1;
    wndRect.right = wndRect.left + BUTTON_WIDTH;
    m_response.Create("",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
    										wndRect,this,IDC_RESPONSE);
	m_response.LoadBitmaps("WAIT");

	m_IBMScore = 0;
	m_humanScore = 0;

	// Initialize Random Generator
	srand((unsigned)time(NULL));
}

CMainWindow::~CMainWindow()
{
	m_font.DeleteObject();
}

int CMainWindow::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_upperToolBar.Create(this,WS_CHILD|WS_VISIBLE|CBRS_TOP) ||
		!m_upperToolBar.LoadBitmap(IDR_UPPERTOOLBAR) ||
		!m_upperToolBar.SetButtons(upperButtons,
		  sizeof(upperButtons)/sizeof(UINT)))

		return -1;		// fail to create

	if (!m_lowerToolBar.Create(this,WS_CHILD|WS_VISIBLE|CBRS_BOTTOM) ||
		!m_lowerToolBar.LoadBitmap(IDR_LOWERTOOLBAR) ||
		!m_lowerToolBar.SetButtons(lowerButtons,
		  sizeof(lowerButtons)/sizeof(UINT)))

		return -1;		// fail to create

	return 0;
}


// ***********
//  GAME Menu
// ***********

void CMainWindow::OnGameRestart()
{
	m_IBMScore = 0;
	m_humanScore = 0;
	
	UpdateScoring();
}

void CMainWindow::OnGameExit()
{
	KillTimer(MORRA_TIMER);
	PostMessage(WM_CLOSE);
}


// ***********
//  MOVE Menu
// ***********

void CMainWindow::OnPaper()
{
	m_humanMove.LoadBitmaps("PAPER");
	m_humanMove.RedrawWindow();
	IBMToMove(PAPER);	
}

void CMainWindow::OnScissors()
{
	m_humanMove.LoadBitmaps("SCISSORS");
	m_humanMove.RedrawWindow();
	IBMToMove(SCISSORS);	
}

void CMainWindow::OnStone()
{
	m_humanMove.LoadBitmaps("STONE");
	m_humanMove.RedrawWindow();
	IBMToMove(STONE);	
}



// ***********
//  HELP Menu
// ***********

void CMainWindow::OnHelpAbout()
{
	CDialog about("AboutBox",this);
	about.DoModal();
}

void CMainWindow::OnHelpRules()
{
	CHelp dlg;
	dlg.DoModal();
}



// *********
//  Playing 
// *********

void CMainWindow::IBMToMove(MOVES humanMove)
{
	// IBM Moves
	MOVES myMove;
	float move = rand()/((float)RAND_MAX);
	
	if (move<0.33)
		myMove = PAPER;
	else       
		if (move<0.66)
			myMove = SCISSORS;
		else
			myMove = STONE;	

	// Who won?
	int status;		// 0 = Win   1 = Draw   2 = Loose
	switch (myMove)
	{
		case PAPER		:	m_IBMMove.LoadBitmaps("PAPER");
							m_IBMMove.RedrawWindow();
							if (humanMove==PAPER)
								status = 1;
							else
								if (humanMove==SCISSORS)
									status = 2;
								else
									status = 0;	
							break;

		case SCISSORS	:	m_IBMMove.LoadBitmaps("SCISSORS");
							m_IBMMove.RedrawWindow();
							if (humanMove==PAPER)
								status = 0;
							else
								if (humanMove==SCISSORS)
									status = 1;
								else
									status = 2;	
							break;

		case STONE		:	m_IBMMove.LoadBitmaps("STONE");
							m_IBMMove.RedrawWindow();
							if (humanMove==PAPER)
								status = 2;
							else
								if (humanMove==SCISSORS)
									status = 0;
								else
									status = 1;	
							break;
	}
	
	// Response
	switch (status)
	{
		case 0	:	m_response.LoadBitmaps("WIN");
					m_response.RedrawWindow();
					m_IBMScore++;
					break;
		
		case 1	:	m_response.LoadBitmaps("DRAW");
					m_response.RedrawWindow();
					break;
		
		case 2	:	m_response.LoadBitmaps("LOOSE");
					m_response.RedrawWindow();
					m_humanScore++;
					break;
	}
	
	UpdateScoring();

	// Wait 1 second
	if (SetTimer(MORRA_TIMER,1000,NULL)==0)
	{
		DWORD time = GetTickCount() + 1000;
		while (GetTickCount()<time)
			;
		
		// Restore Bitmaps	
		m_IBMMove.LoadBitmaps("EMPTY");
		m_IBMMove.RedrawWindow();
		m_humanMove.LoadBitmaps("EMPTY");
		m_humanMove.RedrawWindow();
		m_response.LoadBitmaps("WAIT");
		m_response.RedrawWindow();    
	}

	if (m_humanScore==10)
	{
		CResponse dlg;
		dlg.m_response = "Well, you won...";
		
		if (dlg.DoModal()==IDOK)
		{
			m_IBMScore = 0;
			m_humanScore = 0;
			UpdateScoring();
		}	
		else
			OnGameExit();			
	}
	
	if (m_IBMScore==10)
	{
		CResponse dlg;
		dlg.m_response = "Yeah, I won!";
		
		if (dlg.DoModal()==IDOK)
		{
			m_IBMScore = 0;
			m_humanScore = 0;
			UpdateScoring();
		}	
		else
			OnGameExit();			
	}
}




// **********
//  Painting 
// **********

void CMainWindow::UpdateScoring()
{
	CClientDC dc(this);

	// Write Names & Scores        
 	CString score;
 	char buffer[4];
	CRect wndRect, scoreRect;

	CFont* pOldFont = dc.SelectObject(&m_font);
	COLORREF oldTextColor = dc.SetTextColor(BLACK);
	COLORREF oldBkColor = dc.SetBkColor(GREY);
	UINT oldAlign = dc.SetTextAlign(TA_CENTER | TA_TOP);

	// Computer Name
	m_IBMMove.GetWindowRect(wndRect);
	ScreenToClient(wndRect);

	dc.TextOut(wndRect.left + BUTTON_WIDTH/2,
				wndRect.top - 2*LINE_SPACING,
				IBM_NAME,strlen(IBM_NAME));

	// Computer Score
	score = "  ";
	score += _itoa(m_IBMScore,buffer,10);       
	score += "  ";
	dc.TextOut(wndRect.left+BUTTON_WIDTH/2,wndRect.top-LINE_SPACING,score);
 
	// Human Name
	m_humanMove.GetWindowRect(wndRect);
	ScreenToClient(wndRect);

	dc.TextOut(wndRect.left + BUTTON_WIDTH/2,
				wndRect.top - 2*LINE_SPACING,
				HUMAN_NAME,strlen(HUMAN_NAME));
	
	// Human Score
	score = "  ";
	score += _itoa(m_humanScore,buffer,10);       
	score += "  ";
	dc.TextOut(wndRect.left+BUTTON_WIDTH/2,wndRect.top-LINE_SPACING,score);

	// Restore DC Parameters
	dc.SelectObject(pOldFont);
	dc.SetTextColor(oldTextColor);
	dc.SetBkColor(oldBkColor);
	dc.SetTextAlign(oldAlign);
}

void CMainWindow::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	CRect greyArea;
	GetClientRect(greyArea);

	// Select a grey Pen & Brush
	CPen greyPen;
	greyPen.CreatePen(PS_SOLID,1,GREY);
	CPen* pOldPen = dc.SelectObject(&greyPen);
	CBrush greyBrush;
	greyBrush.CreateSolidBrush(GREY);
	CBrush* pOldBrush = dc.SelectObject(&greyBrush);
	
	// Paint the Background grey
	dc.Rectangle(greyArea);	
	
	// Draw the Bitmaps      
	m_IBMMove.RedrawWindow();
	m_humanMove.RedrawWindow();
	m_response.RedrawWindow();
	
	// Draw the ToolBars
	m_upperToolBar.RedrawWindow();
	m_lowerToolBar.RedrawWindow();
	
	// Update Scoring
	UpdateScoring();
	
	// Nothing to Draw
	ValidateRect(greyArea);	

	// Destroy GDI Objects
	dc.SelectObject(pOldPen);
	greyPen.DeleteObject();
	dc.SelectObject(pOldBrush);
	greyBrush.DeleteObject();
}

void CMainWindow::OnTimer(UINT nIDEvent)
{
	if (nIDEvent==MORRA_TIMER)
	{
		// Restore Bitmaps	
		m_IBMMove.LoadBitmaps("EMPTY");
		m_IBMMove.RedrawWindow();
		m_humanMove.LoadBitmaps("EMPTY");
		m_humanMove.RedrawWindow();
		m_response.LoadBitmaps("WAIT");
		m_response.RedrawWindow();    
		
		KillTimer(MORRA_TIMER);
	}
	else
		CFrameWnd ::OnTimer(nIDEvent);
}

// CMainWindow message map:
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	//{{AFX_MSG_MAP( CMainWindow )
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_COMMAND(ID_GAME_EXIT, OnGameExit)
	ON_COMMAND(ID_GAME_RESTART, OnGameRestart)
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_COMMAND(ID_HELP_RULES, OnHelpRules)
	ON_COMMAND(ID_PAPER, OnPaper)
	ON_COMMAND(ID_SCISSORS, OnScissors)
	ON_COMMAND(ID_STONE, OnStone)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////
// CHelp dialog

CHelp::CHelp(CWnd* pParent /*=NULL*/)
	: CDialog(CHelp::IDD, pParent)
{
	//{{AFX_DATA_INIT(CHelp)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void CHelp::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHelp)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CHelp, CDialog)
	//{{AFX_MSG_MAP(CHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHelp message handlers

BOOL CHelp::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_helpBmp.AutoLoad(IDC_HELP,this);	
	return TRUE;
}





/////////////////////////////////////////////////////////////////////////////
// CResponse dialog

CResponse::CResponse(CWnd* pParent /*=NULL*/)
	: CDialog(CResponse::IDD, pParent)
{
	//{{AFX_DATA_INIT(CResponse)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT    
	
	m_response = "";
}

void CResponse::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResponse)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CResponse, CDialog)
	//{{AFX_MSG_MAP(CResponse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResponse message handlers

BOOL CResponse::OnInitDialog()
{
	CDialog::OnInitDialog();

	GetDlgItem(IDC_WINNER)->SetWindowText(m_response);	
	return TRUE;
}

